//
// Created by nh18u590 on 21.03.2023.
//
// Ph Reynier 13/07/2023
// Functional test 4 for Commode 
// Develop on the base of D5 (ISA TN-02v4-2022): Software Verification Plan for Coma Model for 
// Comet Interceptor
//
// Functional test 4 for testing trajectory. Trajectory has to be compatible with the maximum calculation volume
// REQ-SOFT-01, and the different inputs have to be properly defined (REQ-INT-07)  (from D2:
// User and Software requirements definition document)

#include "utils.h"
#include <iostream>
#include <math.h>
// =================================================================================== //


int trajectory_test(){

    // test1 (acceptance1) checks if the parameters are in their valid ranges
    // test2 (acceptance2) checks if the trajectory is in the grid
    // acceptance is true (0) if both tests are true (O)
    int acceptance = 1;
    int acceptance1 = 1;
    int acceptance2 = 1;

    int gridsize_x = 100;
    int gridsize_y = 100;
    int gridsize = 100;
    int gridsize_apex = 50;
    int cellsize = 100000;

    double d_ca = 200.;
    double phase_angle_of_approach = 10.;
    double theta_sc = 10.;
    double speed = 20.;

    //checks if all parameters are in their valid ranges
    if ((d_ca >= 100) && (d_ca <= 5000)){
        if ((phase_angle_of_approach >= -90) && (phase_angle_of_approach <= 90)){
            if ((theta_sc >= -90) && (theta_sc <= 90)){
                if ((speed >= 10) && (speed <= 80)){
                    std::cout << "Correct parameter ranges for trajectory";
                    acceptance1 = 0;
                }
            }
        }
    }


    int max_distance = (gridsize_x-gridsize_apex)*cellsize;
    double start_time = double(max_distance)/speed;
    double s_ca;
    
    double theta = theta_sc*3.14159265359/180.0;
    s_ca = core::calculate_s_ca(theta_sc, phase_angle_of_approach, d_ca);
    double delta_l = cellsize/20.0;
    double start_dist = -max_distance;
    int number_of_steps = abs(round(start_dist*2.0/delta_l));
    double d;
    double R;
    double S;

    for (int i = 0; i < number_of_steps; i++) {
    
        core::trajectory_sc(i, theta_sc, phase_angle_of_approach, d_ca, s_ca, start_dist, delta_l, &d, &R, &S);

        if ((abs(S) <= gridsize_x * cellsize) && (abs(S) <= gridsize_apex * cellsize)) {
            if (sqrt(R * R - S * S) <= gridsize_y * cellsize) {
                acceptance2 = 0;
                std::cout << "Trajectory fits with the maximum calculation volume ";
		break;
            }
        }
    }

    if (acceptance1 == 0 && acceptance2 == 0){
        acceptance = 0;
    }

    return acceptance;

}



// =================================================================================== //
//standard main function. Editing just the exception message.
int main(int argc, char *argv[])
{
    int val = 1;

    try {
        val = trajectory_test();
    } catch (std::exception &e) {
        std::cout << "test_4 exited with an error of type : " << e.what() << std::endl;
        return 1;
    }
    
    return val;
}

