//
// Created by Philippe Reynier on 30.03.2023.
// Functional test 5 for Commode 
// Develop on the base of D5 (ISA TN-02v4-2022): Software Verification Plan for Coma Model for 
// Comet Interceptor
//
// Functional test 5 for testing Dust Production Rate to fulfill requirement REQ-SOFT-09 (from D2:
// User and Software requirements definition document)
//

#include "utils_fountain.h"
#include <iostream>
#include <tuple>
#include <math.h>
#include <fstream>
#include <random>
#include <string>
#include <vector>
#include "utils.h"


int prod_rate_test(){

    // test (acceptance1) checks if dust production rate is higher than 0 and lower than 10^6
    // test (acceptance2) checks if the power law index is in between -3.7 and -4.2
    // acceptance is true if both tests are true
    int acceptance = 1;
    int acceptance1 = 1;
    int acceptance2 = 1;
    double afrho = 1000.;
//    double phase_function;
//    double Q_sca;
    double power_law_index = -3.6;
    int number_of_bins = 3;
    
    //checks if power_law_index is in the valid range
// Values initially given by NH    
    if ((power_law_index >= -4.5) && (power_law_index <= -3.5)){
// New values: email from NH on 06/06/2023
//    if ((power_law_index >= -4.5) && (power_law_index <= -3.5))
          
          std::cout << "Power law index within range" << std::endl;
          acceptance1 = 0;
    }
    else {
        std::cout << "Power law index outside range" << std::endl;
	return 1;
    }

    //checking dust production rate
    double qd;
    std::vector<double> size_bins(number_of_bins);
    std::vector<double> particle_densities(number_of_bins);
    std::vector<double> terminal_velocities(number_of_bins);
    std::vector<double> phase_function(number_of_bins);
    std::vector<double> Q_sca(number_of_bins);

    size_bins[0]= 0.01;
    size_bins[1]= 0.001;
    size_bins[2]= 0.0001;

    for (int i = 0; i < number_of_bins; i++){
        particle_densities[i] = 533.;
        terminal_velocities[i] = 50.;
        phase_function[i] = 50.;
        Q_sca[i] = 1000.;
    }

    qd = core::estimate_production_rate(afrho, power_law_index, size_bins, particle_densities, terminal_velocities, Q_sca, phase_function);

    if ((qd > 0.) && (qd <= 1.e+6)){
        acceptance2 = 0;
        std::cout << "Dust production rate = " <<  qd << std::endl;

         }
    else {
        std::cout << "Dust production rate outside range" << std::endl;
    }

    if ((acceptance1==0) && (acceptance2==0)){
        acceptance = 0;
    }

    return acceptance;

}




// =================================================================================== //
//standard main function. Editing just the exception message.
int main(int argc, char *argv[])
{
    int val = 1;

    try {
        val = prod_rate_test();
    } catch (std::exception &e) {
        std::cout << "test_5 exited with an error of type : " << e.what() << std::endl;
        return 1;
    }
    
    return val;
}

