//
// Created by Philippe Reynier on 06.06.2023.
// Functional test 6 for Commode 
// Develop on the base of D5 (ISA TN-02v4-2022): Software Verification Plan for Coma Model for 
// Comet Interceptor
//
// Functional test 7 for Scattering Properties linked torequirement REQ-MODEL-12 (from D2:
// User and Software requirements definition document)
//
// For this test it is proposed to check the value obtained for a phase function of 90°, a particle size of 10e-3.11, a species index of 2, the obtained value should be 0.0160103.

#include "utils.h"
#include <math.h>
#include <vector>
#include <iostream>

int scat_prop(){

    // test (acceptance) checks if the correct value is obtained for scattering properties
    // acceptance is equal to 1 if this is the case
    int acceptance = 1;
 
    // 
    int phase_angle = 90.;
    int species_index = 2;
    int number_of_bins = 1;
    std::vector<double> size_bins(number_of_bins);
    size_bins[0] = pow(10.,-3.11);
    double scatresult = 0.0160103;
    std::vector<std::vector<double>> QData(4, std::vector<double>(number_of_bins));    
//
// calculation of phase function
//
    QData = core::get_scattering_properties(size_bins, phase_angle, species_index);
    std::cout << "Computed phase function = "  << QData[3][0] << std::endl;
    std::cout << "Theoretical value = 0.0160103  "  << std::endl;
    //
    // check if scatresult and Qdata(3) (the phase function) are equal
    //
    if ((scatresult - QData[3][0]) <= 0.001){
        acceptance = 0;}
    else {
        std::cout << "error on scattering properties" << std::endl;
	return 1;
    }

    return acceptance;

}




// =================================================================================== //
//standard main function. Editing just the exception message.
int main(int argc, char *argv[])
{
    int val = 1;
 
    try {
        val = scat_prop();
    } catch (std::exception &e) {
        std::cout << "test_7 exited with an error of type : " << e.what() << std::endl;
        return 1;
    }
    
    return val;
}

