//
// Created by Philippe Reynier on 07.06.2023.
// Functional test 8 for Commode 
// Develop on the base of D5 (ISA TN-02v4-2022): Software Verification Plan for Coma Model for 
// Comet Interceptor
//
// Functional test 8 for Outflow Velocity linked torequirement REQ-MODEL-02 and REQ-SOFR-07 (from D2:
// User and Software requirements definition document)
//
// For this test it is proposed to check the values for the largest and lowest bulk velocities


#include "utils.h"
#include <math.h>
#include <vector>
#include <iostream>

int outflow_vel(){

    // test (acceptance) checks if the correct value is obtained for scattering properties
    // acceptance is equal to 0 if this is the case
    int acceptance = 1;
    int acceptance1 = 1;
    int acceptance2 = 1;
 
    // 
    double smallest_size_bin = pow(10.,-8);
    double largest_size_bin = 1.;
    double max_vel = 942.731;
    double min_vel = 0.931823;

    double Q_gas = 15000.;
    double radius_N = 5000.;
    double particle_density = 533.; 
    double nucleus_temp = 317.;
    double vd;

//
// calculation of max velocity
//
    vd =core::calculate_vd(Q_gas, radius_N, particle_density, smallest_size_bin, nucleus_temp);
    std::cout << "Computed highest velocity = " << vd << std::endl;
    std::cout << "Highest velocity value = 942.731"  << std::endl;
    //
    // check if result and max_vel are equal within a tolerance of 10-3
    //
    if (abs(max_vel - vd) <= 0.001){
        acceptance1 = 0;}
    else {
        std::cout << "error on largest velocity" << std::endl;
	return 1;
    }

     vd = core::calculate_vd( Q_gas, radius_N, particle_density, largest_size_bin, nucleus_temp);
     std::cout << "Computed lowest velocity = " << vd << std::endl;
     std::cout << "Lowest velocity value = 0.931823" << std::endl;
    //
    // check if result and max_vel are equal within a tolerance of 10-3
    //
    if (abs(min_vel - vd) <= 0.001){
        acceptance2 = 0;}
    else {
        std::cout << "error on lowest velocity" << std::endl;
	return 1;
    }

    //
    // 
    //

    if (acceptance1 == 0 && acceptance2 == 0){
        acceptance = 0;
    }

    return acceptance;

}




// =================================================================================== //
//standard main function. Editing just the exception message.
int main(int argc, char *argv[])
{
    int val = 1;

    try {
        val = outflow_vel();
    } catch (std::exception &e) {
        std::cout << "test_8 exited with an error of type : " << e.what() << std::endl;
        return 1;
    }
    
    return val;
}

