
/*
    unique header for core-API python bindings
    RA 2/3/2023
*/
#define BOOST_PYTHON_MAX_ARITY 30
#include <boost/python.hpp>
#include <boost/python/suite/indexing/vector_indexing_suite.hpp>
#include <grid.h>
#include <run_models.h>
#include <utils.h>
#include <utils_fountain.h>


BOOST_PYTHON_MODULE(PyComMoDE) {

    using namespace boost::python;

    //alternative using vector_indexing
    class_< std::vector<double> > ("DoubleVector")
        .def(boost::python::vector_indexing_suite< std::vector<double> >());
    class_< std::vector< std::vector<double> > > ("DoubleMatrix")
        .def(boost::python::vector_indexing_suite< std::vector< std::vector<double> > >());
    class_< std::vector<int> > ("IntVector")
        .def(boost::python::vector_indexing_suite< std::vector<int> >());

    class_<core::ComaHeader> ("ComaHeader")
        .add_property("gridsize_x", &core::ComaHeader::gridsize_x)
        .add_property("gridsize_y", &core::ComaHeader::gridsize_y)
        .add_property("cellsize", &core::ComaHeader::cellsize)
        .add_property("gridsize_apex", &core::ComaHeader::gridsize_apex)
        .add_property("number_of_bins", &core::ComaHeader::number_of_bins)
        .add_property("power_law_index_used", &core::ComaHeader::power_law_index_used)
        .add_property("power_law_index_passed", &core::ComaHeader::power_law_index_passed)
        .add_property("lower_size_limit", &core::ComaHeader::lower_size_limit)
        .add_property("higher_size_limit", &core::ComaHeader::higher_size_limit)
        .add_property("particle_density", &core::ComaHeader::particle_density);

    //detailed internal calls 
    def("get_size_bins", &core::get_size_bins);
    def("calc_particle_density", &core::calc_particle_density);
    def("get_terminal_v", &core::get_terminal_v);
    def("get_scattering_properties", &core::get_scattering_properties);
    def("estimate_production_rate", &core::estimate_production_rate);
    def("calculate_beta", &core::calculate_beta);
    def("get_size_dependent_qd", &core::get_size_dependent_qd);
    def("run_numerical_model", &core::run_numerical_model);
    def("numerical_model_sampling", &core::numerical_model_sampling);
    def("prepareSmoothingWeights", &core::prepareSmoothingWeights);
    def("smoothingValue", &core::smoothingValue);
    def("calc_afrho", &core::calc_afrho);
    def("write_file", &core::write_file);
    def("run_fountain", &core::run_fountain);
    def("write_header", &core::write_header);
    def("read_header", &core::read_header);
    def("read_grid", &core::read_grid);
    def("mass_along_trajectory", &core::mass_along_trajectory);
    def("write_header_spacecraft", &core::write_header_spacecraft);
    def("initializeGrid", &core::initializeGrid);
    
}
