//
// Created by nh18u590 on 29.06.2022.
// Modified by RArpa_OPT on 26.2.2023
//

#ifndef UNTITLED_RUN_MODELS_H
#define UNTITLED_RUN_MODELS_H

#include <cstdint>
#include <math.h>
#include <string> 
#include "utils.h"
#include "utils_fountain.h"


/*!
    @mainpage ComMoDE model documentation

    A detailed explanation of ComMoDE model is reported in the pdf document
   here: <a href="Description_of_ComMoDE_v2.pdf"> ComMoDE.pdf</a>

*/


using namespace std;
namespace core{

void numerical_model_sampling(
    int gridsize_x, int gridsize_y, int gridsize_apex, int cellsize, 
    double beta, int size_index, double particle_size, double v0, 
    double particle_density, double size_dependent_qd, double Q_sca, double phase_function, 
    double amplitude_rot, double period_rot, int angular_dependency_index, double cosine_scaling,
    std::vector<std::vector<double>>& grid);

double run_numerical_model(std::vector<std::vector<double>> &grid, int gridsize_x, int gridsize_y, int cellsize, int gridsize_apex, 
                         double beta, int size_index, double particle_size, double v0, double particle_density, 
                         double size_dependent_qd, double Q_sca, double phase_function, 
                         double amplitude_rot, double period_rot, int angular_dependency_index, double cosine_scaling,
                         const std::string &dumpModelDirectoryPath);


double run_fountain(std::vector<std::vector<double>> &grid, int gridsize_x, int gridsize_y, int cellsize, int gridsize_apex, 
                  double beta, double qd, double particle_size, double v0, int size_index, double particle_density, 
                  double Q_sca, double phase_function, const std::string & dumpModelDirectoryPath);

void run_model_all_sizes(int gridsize_x, int gridsize_y,int cellsize,int gridsize_apex,
                         int number_of_bins, double particle_density, double lower_size_limit, double higher_size_limit, double power_law_index,
                         double Q_gas, double radius_N,double nucleus_temp, int species_index, int phase_angle, double heliocentric_distance,
                         int day_night_asym_switch, int angular_dependency_index, double cosine_scaling,
                         int rotational_switch, double amplitude_rot, double period_rot,
                         int qd_estimation_switch, double qd_input, double afrho_input,
                         const std::string & dumpModelDirectoryPath, int &counterOfProcessedBins);

void run_trajectory_module(double d_ca, double phase_angle_of_approach, double theta_sc, 
                           double magnitude_of_velocity, double spacecraft_frontal_area,
                           const std::string &restoreModelDirectoryPath, 
                           const std::string &resultsOutputDirectoryPath);


} // end namespace core

#endif //UNTITLED_RUN_MODELS_H
