//
// Created by nh18u590 on 17.02.2022.
//

#ifndef UNTITLED_UTILS_H
#define UNTITLED_UTILS_H

#include <cstdint>
#include <math.h>
#include <vector>
#include <string>

#define G 6.67408*pow(10,-11)
#define M_sun 1.989*pow(10,30)
#define dist_sun 1.496*pow(10,11)
#define pi 3.14159265359
#define sqrt2 1.41421356237
#define lightspeed 299792458
#define L_sun 3.828*pow(10,26)
#define g_const M_sun*G/(dist_sun*dist_sun)
#define avogadro 6.022*pow(10,23)


using namespace std;

namespace core{
void initializeGrid(int x, int y, std::vector<std::vector<double>> & data);
void velocity_distribution(double v0, double *v, double emission_angle , int angular_function_index, double cosine_scaling);
void get_qd_distribution(double *qd_distribution, double emission_angle, int angular_function_index, double cosine_scaling);

void trajectory(int *x, int *y, double *time, double *emission_angle, double *beta, double *v, double *v_temp, int cellsize);
void get_time_step(double *time_step, double *v_temp, int cellsize);

void get_emission_angle(double* emission_angle,double* solid_angle_scaling, int i, int number_of_trajectories);

void write_file(const vector<vector<double>> &grid, const std::string &filename);
void cell_volume(double* volume, int y, int cellsize);
void calc_endtime(double v0, double *max_time, double beta, int cellsize);
void rotational_variablility(double amplitude, double period, double *scaling_rotational,double *time);
void scale_to_qd(double* scaling_qd, double qd_per_size, double* particle_mass);

double weights_smoothing(int i, int j, double sigma);

void trajectory_sc(int i, double theta, double phase_angle_of_approach, double d_ca, double s_ca, double start_dist, double delta_l, double* d, double* R, double* S);
void get_dust_density_index(int* x_position, int* y_position,double delta_l, double R, double S, int cellsize, int gridsize_apex);
double calculate_s_ca(double theta, double phase_angle_of_approach, double d_ca);

double calculate_iv(double Q_gas, double radius_N, double particle_density, double particle_size, double nucleus_temp);
double calculate_vd(double Q_gas, double radius_N, double particle_density, double particle_size, double nucleus_temp);

std::vector<int> prepareSmoothingWeights(
    int cellsize,
    std::vector<std::vector<double>> & weights1, 
    std::vector<std::vector<double>> & weights2, 
    std::vector<std::vector<double>> & weights3, 
    std::vector<std::vector<double>> & weights4);


double smoothingValue(
    int x, int y, int gridsize_x, int gridsize_y, int gridsize_apex, int cellsize,
    const std::vector<std::vector<double>> & grid,
    std::vector<int> & smoothing_radius,
    std::vector<std::vector<double>> & weights1, 
    std::vector<std::vector<double>> & weights2, 
    std::vector<std::vector<double>> & weights3, 
    std::vector<std::vector<double>> & weights4 );

std::vector<std::vector<double>> perform_smoothing(const vector<vector<double>>&grid, int gridsize_x, int gridsize_y, int cellsize, int gridsize_apex, 
                                                   int size_index, double particle_size, double particle_density, double &afrho_calculated, double Q_sca, 
                                                   double phase_function, const std::string & dumpModelDirectoryPath);

double calculate_frho(const vector<vector<double>> &grid, int gridsize_x, int gridsize_y, int cellsize, int gridsize_apex);
double calc_afrho(const vector<vector<double>> &grid_smooth , int gridsize_x, int gridsize_y,int cellsize, int gridsize_apex, double Q_sca, double phase_function, double particle_size);

double estimate_production_rate(double afrho, double power_law_index, const std::vector<double> &size_bins, 
                                const std::vector<double> &particle_densities, const std::vector<double> &terminal_velocities,
                                const std::vector<double> & Q_sca, const std::vector<double> &phase_function);

std::vector<double> calc_particle_density(const std::vector<double> &particle_sizes, double particle_density);
std::vector<double> calculate_beta(double heliocentric_distance, const std::vector<double> &size_bins, const std::vector<double> &particle_densities, 
                                   const std::vector<double> &Q_pr, const std::vector<double> &phase_function);

std::vector<double> get_size_bins( int number_of_bins, double lower_limit_exp, double higher_limit_exp);

std::vector<double> get_size_dependent_qd( double qd, double power_law_index, const std::vector<double> &size_bins, 
                                           const std::vector<double> &particle_densities, const std::vector<double> &terminal_velocities);

std::vector<double> get_terminal_v(const std::vector<double> &size_bins, const std::vector<double> &particle_densities, double Q_gas, double radius_N,double nucleus_temp);

double mass_along_trajectory(vector<vector<double>>&grid_smooth, int gridsize_x, int gridsize_y, double particle_density, 
                             double particle_size, int cellsize, int gridsize_apex,int size_index ,double d_ca, 
                             double phase_angle_of_approach, double theta, double speed, double scarea, const std::string &outputResultsDirPath);

void write_sc_output(const vector<double> &dust_density_of_time,const vector<double> &flux , 
                     const vector<double> &fluence, const vector<double> &R_coordinates, 
                     const vector<double> &S_coordinates, const vector<double> &time, const std::string &filename);

void read_files(double *q_sca,double *q_abs, double *q_pr, double *phase_function, int phase_angle, double exponent, int species_index);
void interpolate(double &Q_abs, double &Q_sca, double &Q_pr,double &phase_function, int phase_angle, double particle_size, int species_index);
std::vector<std::vector<double>> get_scattering_properties(const std::vector<double> &size_bins, int phase_angle, int species_index);

void write_header(int gridsize_x, int gridsize_y, int cellsize, int gridsize_apex,double particle_density, 
                  int lower_size_limit, int higher_size_limit,double power_law_index_passed, double Q_gas,double radius_N,
                  double nucleus_temp,int species_index,double heliocentric_distance,int day_night_asym_switch,
                  int angular_dependency_index,double cosine_scaling, int rotation_switch, double amplitude_rot, 
                  double period_rot, int qd_estimation_switch, double qd,double afrho_input, const vector<double> &size_bins, 
                  const vector<double> &terminal_velocities, const vector<double> &beta, const std::vector<vector<double>> &Qdata, int phase_angle, 
                  const vector<double> &size_dependent_qd,double total_afrho_calculated, const std::string &storeDirectoryPath);

void write_header_spacecraft(double d_ca, double phase_angle_of_approach, double theta_sc, double magnitude_of_velocity, 
                             double spacecraft_frontal_area,int number_of_bins, const vector<double> &size_bins, 
                            const vector<double> &impact_mass_per_bin, double total_mass, const std::string &outputResultsDirPath);


/**!
    \struct ComaHeader
    \brief public collection of essential primitive input data for restoring structures of coma simulation results from dumped data 
*/
struct ComaHeader
{
public: 
    int gridsize_x; /**< x-direction grid size*/
    int gridsize_y;/**< y-direction grid size*/
    int cellsize; /**< grid resolution*/
    int gridsize_apex; /**< apex of grid in x direction*/
    int number_of_bins; /**< number of particle bins in size distribution*/
    double power_law_index_used; /**< powerlaw index value effectively used in size distribution: typically equal to -1*power_law_index_passed*/
    double power_law_index_passed; /**< powerlaw index of size distribution as passed by the user*/
    double lower_size_limit; /**< exponent x of 10^x to identify the smallest size particle in size distribution*/
    double higher_size_limit; /**< exponent x of 10^x to identify the largets size particle in size distribution*/
    double particle_density; /**< typical density of the dust particle*/     
};

ComaHeader read_header(const std::string &headerDirPath);

vector<vector<double>> read_grid(int gridsize_x, int gridsize_y, int size_index, const std::string &modelDataDirPath);

double integrate_bin_vol(double particle_size_min, double particle_size_max, double power_law_index);

} // end namespace core

#endif //UNTITLED_UTILS_H
