#The current script will (download with wget, bu sure to have internet connection and capacity
# to access the external network) and install boost 1.78.0 in prefix 
# <currentdir>/boost-ESAComMoDE with library sub modules: 
# boost.python

#the building will require a minor version of python between 3.8.x and 3.10.x. The script will ask to pass 
#the full path of the correct python interpreter to use as argument (usually inside 
#C:\Users\<youruser>\AppData\Local\Programs\Python\PythonXYY). f.e.
#.\buildBoost C:\Users\johndoe\AppData\Local\Programs\Python\Python310\python.exe" to pick up a 3.10 version.  
#(windows interpreter is always a python.exe inside a PythonXYY that specify 
#always the major(X).minor(YY) version of the interpreter).  

if ($null -eq $args[0]) {
    Write-Host "Error : no path to python interpreter(within 3.8 - 3.10) is provided. 
    If you have a suitable version of python installed please re-run this script 
    providing the full path to a python interpreter command as argument. 
    For example, for a python 3.10.x : 
    .\buildBoost.ps1  C:\Users\johndoe\AppData\Local\Programs\Python\Python310\python.exe"
    exit(1)
}
else {
    $PYTHON_COMMAND = $args[0] 
} 

#verify its admissible python
$expr = -join ($PYTHON_COMMAND, " --version")
$version = (Invoke-Expression $expr)
if ( $version -like "*Python*") {
    $numstringarray = ([regex]::Matches($version, '[0-9]+').value).split(" ")
    $numstring = [float]::Parse($numstringarray[1])
}
else { 
    Write-Host "Error $PYTHON_COMMAND is not recognized as a valid python interpreter"
    exit(1)
}

if ($numstring -gt 10 -or $numstring -lt 8) {
    Write-Host "Error : $PYTHON_COMMAND command does not match any of 
    recommendend version >= 3.8.x and <= 3.10.x. 
    If you have a suitable version of python installed please re-run this script 
    providing the full path to python.exe interpreter command as argument. 
    For example, for a python 3.10.x : 
    .\buildBoost.ps1 C:\Users\johndoe\AppData\Local\Programs\Python\Python310\python.exe"
    exit(1)
}

$currentdir = $pwd
$boostsourcedir = ".\boost_1_78_0"
$boostpackage = ".\boost_1_78_0.zip"

$unzipdata = -join ("cmd.exe /c ", '"C:\Windows\System32\tar.exe -xf boost_1_78_0.zip"')

#clean up previous boost build
if (Test-Path -Path $boostsourcedir) {
    Remove-Item $boostsourcedir -Recurse
}

#try to download the file via Webrequest
if (-Not(Test-Path -Path $boostpackage -PathType Leaf)) {
    $ProgressPreference = 'SilentlyContinue'
    Invoke-WebRequest "https://boostorg.jfrog.io/artifactory/main/release/1.78.0/source/boost_1_78_0.zip" -OutFile $boostpackage  
}

Invoke-Expression $unzipdata 
if (1 -eq $LASTEXITCODE){ 
    Write-Host "Error while downloading/extracting boost_1_78_0.zip with Web-Request: the archive may be corrupted."
    Write-Host "Please download it manually from https://boostorg.jfrog.io/artifactory/main/release/1.78.0/source . "
    Write-Host "Then, copy it into $currentdir and try again to run the build script" 
    exit(2)
}

Set-Location $boostsourcedir

$projectconfigfile = "./project-config.jam" 
if (Test-Path -Path $projectconfigfile -PathType Leaf) {
    Remove-Item -Path $projectconfigfile
}
$bootstrap_command = ".\bootstrap.bat"
Invoke-Expression $bootstrap_command

$checkPythonRow = (Select-String -Path .\project-config.jam -Pattern 'using python ')
if ($null -eq $checkPythonRow) {
    $pythonstring = -join ("using python ;")
    Add-Content project-config.jam $pythonstring
}

$userconfigfile = "user-config.jam";
$userconfigfilePath = -join (".\", "user-config.jam");
if (Test-Path -Path $userconfigfilePath -PathType Leaf) {
    Remove-Item -Path $userconfigfilePath
}
#sanitize backslash in normal slash in the path
$pattern = '[\\]'
$sanit_arg = $PYTHON_COMMAND -replace $pattern, '/' 
$value = -join ("using python : : ", $sanit_arg, " ;")
New-Item -Path . -Name $userconfigfile -ItemType "file" -Value $value


$install_command = ".\b2 install link=shared --with-python address-model=64 --prefix="
$install_command = -join ($install_command, $currentdir, '"\boost-ESAComMoDE"')
$install_command = -join ($install_command, " --user-config=./user-config.jam")

Invoke-Expression $install_command

Set-Location $currentdir