#!/bin/bash

#The current script will (download with wget, bu sure to have internet connection and capacity
# to access the external network) and install boost 1.78.0 in prefix 
# <currentdir>/boost-ESAComMoDE with library sub modules: 
# boost.python

#the building will require a minor version of python between 3.8.x and 3.10.x. The script will ask to pass 
#the full path of the correct python interpreter to use as argument (usually inside /usr/bin). f.e.
#.\buildBoost /usr/bin/python3.8" to pick up a 3.8 version (linux interpreter will specify 
#always the major.minor version of the interpreter).  

if [ -z $1 ]; then
    echo "Error : no path to python interpreter(within 3.8 - 3.10) is provided. 
    If you have a suitable version of python installed please re-run this script 
    providing the full path to a python interpreter command as argument. 
    For example, for a python 3.8.x : 
    ./buildBoost.sh /usr/bin/python3.8"
    exit 1
else
    PYTHON_COMMAND=$1 
fi 

#verify its admissible python
version=$($PYTHON_COMMAND --version)
if [[ $version == *"Python"* ]]; then
    numstringarray=( $(echo ${version} | grep -o -E "[0-9]+") )
    numstring=${numstringarray[1]}
else 
    echo "Error "$PYTHON_COMMAND" is not recognized as a valid python interpreter"
    exit 1 
fi

if [[ $numstring -gt "10" || $numstring -lt "8" ]]; then 
    echo "Error : $PYTHON_COMMAND command does not match any of 
    recommended version >= 3.8.x and <= 3.10.x. 
    If you have a suitable version of python installed please re-run this script 
    providing the full path to a python3 interpreter command as argument. 
    For example, for a python 3.8.x : 
    ./buildBoost.sh /usr/bin/python3.8"
    exit 1
fi


currentdir=$(pwd)
boostsourcedir="./boost_1_78_0"

#clean up previous boost build
if [ -d "$boostsourcedir" ]; then
    rm -r $boostsourcedir
fi

#try to download the file via wget
if [ ! -f "./boost_1_78_0.tar.gz" ]; then 
    wget -4 https://boostorg.jfrog.io/artifactory/main/release/1.78.0/source/boost_1_78_0.tar.gz
fi

#check if the archive is usable
tarerror=$(tar -tvf boost_1_78_0.tar.gz 2>&1 >/dev/null)
if [ ! -z "$tarerror" ]; then
    echo "Error while downloading boost_1_78_0.tar.gz with wget: the archive may be corrupted."
    echo "Please download it manually from https://boostorg.jfrog.io/artifactory/main/release/1.78.0/source"
    echo "Then, copy it into $currentdir and try again to run the build script" 
    exit 2
fi

# extract archive and move into it.
tar -zxvf boost_1_78_0.tar.gz
cd $boostsourcedir

#run the bootstrap
if [ -f "./project-config.jam" ]; then
    rm project-config.jam
fi
./bootstrap.sh

#fixing the project-config.jam
checkPythonRow=$(grep -rn 'using python ' ./project-config.jam)
if [ -z "$checkPythonRow" ]; then
    echo "using python ;" >> ./project-config.jam
fi

userconfigfilePath="./user-config.jam"
if [ -f "$userconfigfilePath" ]; then
    rm $userconfigfilePath
fi
value="using python : : "$PYTHON_COMMAND" ;"
echo "$value" >> $userconfigfilePath

install_command="./b2 install link=shared --with-python address-model=64 --prefix="
install_command+="$currentdir/boost-ESAComMoDE"
install_command+=" --user-config=./user-config.jam"

#run install command
$install_command

cd $currentdir