import sys, os
import math
sys.path.append('..')
from frontEnd.ComaParameters import ComaParameters
from PySide2.QtCore import Qt
from PySide2.QtWidgets import QApplication
import xml.etree.ElementTree as ET
import xml.dom.minidom as XDOM

#Summary:
#The current test checks if the Coma User Input form embedded in the GUI can:
# - fill correctly itself
# - dump its content to a readable human file (inputComa.xml)
# - restore data from inputComa.xml
# - handle out-of-bounds values passed to its form entries 


#needed to run widget classes of pyside2
app = QApplication()

#overall checker
check = False

comaform = ComaParameters()
# testing filling of Coma input form

try:
    comaform.particleSizeDistro.distroType.setCheckState(Qt.Unchecked)
    comaform.particleSizeDistro.sizeBins.setValue(10)
    comaform.particleSizeDistro.powerLaw.setValue(3.60)
    comaform.particleSizeDistro.minSize.setValue(-7)
    comaform.particleSizeDistro.maxSize.setValue(-1)

    comaform.scattering.helioDist.setValue(1.0)
    comaform.scattering.phaseAngle.setValue(90)
    comaform.scattering.materials.setCurrentIndex(2)

    comaform.dustProduction.chooseAfrho.setCheckState(Qt.Checked)
    comaform.dustProduction.afrho.setValue(25.0)
    comaform.dustProduction.dustProdRate.setValue(30000.0)

    comaform.outflowVelocity.gasProdRate.setValue(15000.0)
    comaform.outflowVelocity.enableExpertWidget.setCheckState(Qt.Checked)
    comaform.outflowVelocity.nucleusRadius.setValue(5000.0)
    comaform.outflowVelocity.nucleusTemperature.setValue(317.0)
    comaform.outflowVelocity.dustBulkDensity.setValue(533.0)

    comaform.dayNightAsymmetry.chooseDNA.setCheckState(Qt.Unchecked)
    comaform.dayNightAsymmetry.angularDepLaw.setCurrentIndex(0) #cosine law
    comaform.dayNightAsymmetry.cosineAmp.setValue(0.0)

    comaform.rotationalVariability.chooseRotation.setCheckState(Qt.Unchecked)
    comaform.rotationalVariability.rotationPeriod.setValue(0.0)
    comaform.rotationalVariability.rotAmplitudeVal.setValue(0.0)

    comaform.gridResolution.cometoDist.setValue(60000) #in Km
    comaform.gridResolution.resolution.setValue(40) #in Km

    check = True
except :
    print("Failed to fill the interface form")
    check = False

# dump to test001_inputcoma.xml file

try: 
    fileinputcoma = os.path.join(os.getcwd(),"test001_inputcoma.xml")
    comatree = ET.Element('root')
    comaform.dumpToXML(comatree)    

    rstring = ET.tostring(comatree, 'utf-8')
    xdom = XDOM.parseString(rstring)
    towritestring = xdom.toprettyxml(indent="    ")
    with open(fileinputcoma, "w") as file:
        file.write(towritestring)

    check = check and os.path.exists(fileinputcoma)
except: 
    print("Failed to dump User input to xml file")
    check = False

# reset coma form to default state and restore User Input from test001_inputcoma.xml
try: 
    comaform.setDefaults()
    comatree = ET.parse(fileinputcoma)
    comaform.restoreFromXML(comatree)    

    #check some input value
    check = check and str(comaform.particleSizeDistro.sizeBins.value()) == '10'
    check = check and str(comaform.scattering.materials.currentIndex()) == '2'

    check = check and "{:.2f}".format(comaform.dustProduction.afrho.value()) == '25.00'
    check = check and "{:.2f}".format(comaform.outflowVelocity.nucleusTemperature.value()) == '317.00'
 
except: 
    print("Failed to restore User input from xml file")
    check = False


# try to set value out of prescribed limits testing sizebins [10,80] and afrho [25.0, 300.0]
try: 
    #push some outbounds values. Classes will automatically set to the nearest admissible value
    #in the interval
    comaform.particleSizeDistro.sizeBins.setValue(8)
    comaform.dustProduction.afrho.setValue(310.0)

    check = check and str(comaform.particleSizeDistro.sizeBins.value()) == '10'
    check = check and "{:.2f}".format(comaform.dustProduction.afrho.value()) == '300.00'
    
except: 
    print("Failed to handle out-of-bounds value for User input")
    check = False

###############################
os.remove(fileinputcoma)

print("interface_functional_test_001 passed... :{}".format(check))

