import sys, os, shutil
sys.path.append('..')
from frontEnd.ParametersTab import ParamTabWidget
from frontEnd.StatusProject import StatusProject
from backEnd.fileFunctions import createNewProject, checkValidProject
from PySide2.QtCore import Qt
from PySide2.QtWidgets import QApplication

import xml.etree.ElementTree as ET
import xml.dom.minidom as XDOM

#Summary: 
#The current test check the creation of a ComMoDe project folder, that should contain in the end
# - databases folder (inherited from the app internal files)
# - coma-results folder to host coma simulation result files
# - fly-results folder to host flyby postprocessing result files
# - inputComa.xml file collecting User Input to set-up the coma-simulation in xml format
# - inputFlyBy.xml file collecting User Input to set-up the flyby post-processing in xml format
# - .status-project file internal app file to track the status of coma and flyby processes.


#needed to run widget classes of pyside2
app = QApplication()

check = bool(False)
# create a brand new default project in ./test002_brandNewProject
mainDatabaseRoot = os.path.normpath(os.path.join(os.getcwd(), "..", "..", "bin"))
mainIconRoot = os.path.normpath(os.path.join(os.getcwd(), "..", "frontEnd", "assets"))


newProjectName = "test002_brandNewProject"

#just in case clean up folders named as newProjectName
pathfolder = os.path.join(os.getcwd(), newProjectName)
if os.path.isdir(pathfolder):
    shutil.rmtree(pathfolder)

check = createNewProject(os.getcwd(), newProjectName, mainDatabaseRoot, logger=None)

if not check :
    print ("Failed to create the project folder")
    print("interface_functional_test_002 passed... :{}".format(check))
    exit()

projectFolder = os.path.join(os.getcwd(), newProjectName)

#adding default inputComa.xml and inputFlyBy.xml
try:
    tabParams = ParamTabWidget(mainIconRoot)
    tabParams.setDefaults()

    fileinputcoma = os.path.join(projectFolder,"inputComa.xml")
    comatree = ET.Element('root')
    tabParams.dumpComaToXML(comatree)    
    rstring = ET.tostring(comatree, 'utf-8')
    xdom = XDOM.parseString(rstring)
    towritestring = xdom.toprettyxml(indent="    ")
    with open(fileinputcoma, "w") as file:
        file.write(towritestring)

    fileinputflyby = os.path.join(projectFolder,"inputFlyBy.xml")
    flybytree = ET.Element('root')
    tabParams.dumpFlyByToXML(flybytree)    
    rstring = ET.tostring(flybytree, 'utf-8')
    xdom = XDOM.parseString(rstring)
    towritestring = xdom.toprettyxml(indent="    ")
    with open(fileinputflyby, "w") as file:
        file.write(towritestring)

except: 
    print ("Failed to create inputComa.xml and inputFlyBy.xml")
    check = False    
    print("interface_functional_test_002 passed... :{}".format(check))
    exit()

#write the .status_project internal file
try: 
    stat = StatusProject()
    stat.writeOnFile(projectFolder)
except: 
    print ("Failed to create .status-project file")
    check = False    
    print("interface_functional_test_002 passed... :{}".format(check))
    exit()


#overall check if the current project folder is a valid project
check = checkValidProject(projectFolder)
print("interface_functional_test_002 passed... :{}".format(check))

shutil.rmtree(projectFolder)